-- A set of different-looking buff bars. Inspired by Elkano's Buff Bars.
-- The player can left-click on a buff to cancel it.

-- Set our default placement values. If a SavedVariable file exists, then these will be overwritten after all our code is loaded. If it doesn't, then these will work as defaults.
WhimseyDots_Placement = {x = 841,y = 646}
WhimseyDots_BarSize = {x = 250, y = 20}
WhimseyDots_BarColorFront = {r = 1, g=0.0, b=0.0, a=0.4}
WhimseyDots_BarColorBack = {r = 1, g=1, b=1, a=0.1}
WhimseyDots_FontSize = {text = 12, timer = 12}
WhimseyDots_FontColor = {r = 1, g=1, b=1, a=1}
WhimseyDots_BlackList = {"Searing Vitality", "Crumbling Resistance"}
-- Create our context so we can do rendering.
local context = UI.CreateContext("HUD")

-- Because it's currently impossible to destroy frames, we keep a reservoir of buff bars around. Buff bars work as a single unified whole, so once they're created, we can just call their "set" function.
local activeBuffBars = {}
local zombieBuffBars = {}

local lastAttach = nil  -- Attachment point for our next buff bar.
local function StartBuffBars()  -- Hide everything we have and start adding bars one at a time.
  for _, v in pairs(activeBuffBars) do
    table.insert(zombieBuffBars, v)
    v:SetVisible(false)
    v:SetPoint("TOPLEFT", UIParent, "TOPLEFT")
  end
  
  activeBuffBars = {}
  lastAttach = nil
end

local function AddBuffBar(buff, time, targetdetails)
  local bar = table.remove(zombieBuffBars)
  if not bar then
    -- We don't have any bars remaining, so we create a new one.
    -- Our Bars are considered single objects that can be dealt with atomically. Each one has the functionality needed to update itself.
    bar = UI.CreateFrame("Frame", "Bar", context)
    bar.td = targetdetails
    bar.targetdetails = Inspect.Unit.Detail(bar.td)	

	--print(bar.td)
	--print(bar.targetdetails.name)
    -- Set an initial vertical pin to make some of our calculations work properly
    bar:SetPoint("TOP", UIParent, "TOP")
   
    bar.text = UI.CreateFrame("Text", "Text", bar)
    bar.timer = UI.CreateFrame("Text", "Timer", bar)
    bar.icon = UI.CreateFrame("Texture", "Icon", bar)
    
    -- Solid background - this is the actual "bar" part of it.
    bar.solid = UI.CreateFrame("Frame", "Solid", bar)
    bar.solid:SetLayer(-1)  -- Put it behind every other element.
    
    bar.text:SetText("test")
    bar.text:SetHeight(bar.text:GetHeight())  -- Set an initial height. We'll be overriding this, we just want to make sure it works.
    
    bar.icon:SetPoint("TOPRIGHT", bar, "TOPLEFT") -- The icon is pinned to the top-left corner of the bar.
    bar.icon:SetPoint("BOTTOM", bar, "BOTTOM") -- Vertically, it always fills the entire bar.
    
    bar.text:SetPoint("TOPLEFT", bar.icon, "TOPRIGHT") -- The text is pinned to the top-right corner of the icon.
    bar:SetPoint("BOTTOM", bar.text, "BOTTOM")  -- The bar is set to always be as high as the text is.
    
    bar.timer:SetPoint("TOPRIGHT", bar, "TOPRIGHT") -- The timer is pinned to the top-right corner.
    
    bar.text:SetPoint("RIGHT", bar.timer, "LEFT") -- Make sure the text doesn't overrun the timer. We'll be changing the text's height based on the contents, but we'll leave the width calculated this way.
    
    -- Set the solid bar to fill the entire buff bar.
    bar.solid:SetPoint("TOPLEFT", bar, "TOPLEFT")
    bar.solid:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT")
    
    -- This is hardcoded, but in a full fleshed-out addon, it would be set by the user.
    -- This could be done now with slash commands, but couldn't be saved yet.

    
    -- The function that actually sets everything.
    function bar:SetBuff(buff, time)
 --     if buff.debuff then
        -- For the sake of this addon, Debuffs are somewhat larger and red-themed.
 --       self:SetBackgroundColor(0.5, 0.3, 0.3, 0.3)
 --      self.solid:SetBackgroundColor(0.5, 0.3, 0.3)
 --       self.text:SetFontSize(20)
 --       self.timer:SetFontSize(20)
 --       self.text:SetHeight(self.text:GetHeight())
 --     else
        -- Buffs are smaller and blue-themed.
		bar:SetWidth(WhimseyDots_BarSize.x)
        self:SetBackgroundColor(WhimseyDots_BarColorBack.r, WhimseyDots_BarColorBack.g, WhimseyDots_BarColorBack.b, WhimseyDots_BarColorBack.a)
        self.solid:SetBackgroundColor(WhimseyDots_BarColorFront.r, WhimseyDots_BarColorFront.g, WhimseyDots_BarColorFront.b, WhimseyDots_BarColorFront.a)
        self.text:SetFontSize(WhimseyDots_FontSize.text)
		self.text:SetFontColor(WhimseyDots_FontColor.r, WhimseyDots_FontColor.g, WhimseyDots_FontColor.b, WhimseyDots_FontColor.a)
        self.timer:SetFontSize(WhimseyDots_FontSize.timer)
		
        self.text:SetHeight(WhimseyDots_BarSize.y)--self.text:GetHeight())
--      end
      
      -- Re-square and set the icon
      self.icon:SetWidth(self.icon:GetHeight())
      self.icon:SetTexture("Rift", buff.icon)
      
      -- Display our stacking multiple.
      if buff.stack then
        self.text:SetText(buff.name .. " x" .. buff.stack)
      else
        self.text:SetText(buff.name)
      end
      
      if buff.duration then
        self.completion = buff.begin + buff.duration
        self.duration = buff.duration
        
        -- Display everything we might have hidden.
        self.solid:SetVisible(true)
        self.timer:SetVisible(true)
        
        self:Tick(time)
      else
        self.completion = nil
        
        -- This is a permanent buff without a timer, so don't show any of that.
        self.solid:SetVisible(false)
        self.timer:SetVisible(false)
        self.timer:SetWidth(0)
      end
      
      self.debuff = buff.debuff
      self.buffid = buff.id
    end
    
    -- This is our update function, called once every frame.
    function bar:Tick(time)
      if self.completion then
        local remaining = self.completion - time
        --bar.targetdetails = Inspect.Unit.Detail(bar.td)
		--print(bar.targetdetails.name)
        if bar.targetdetails.health == 0 or remaining < 0 then
		  --print("Kill timer on ", bar.targetdetails.name, " with ", bar.targetdetails.health)
          self.timer:SetVisible(false)
        else
          -- Update our timer.
          self.solid:SetPoint("RIGHT", bar, remaining / self.duration, nil)
          -- Rift's UI system allows you to pin points to an arbitrary place on a frame, not just the corners and edges.
          -- These two lines are equivalent:
          -- a:SetPoint("TOPLEFT", b, "BOTTOMLEFT")
          -- a:SetPoint(0, 0, b, 0, 1)
          -- Coordinates are passed in X, Y order, with 0 being the topleft and 1 being the bottomright.
          -- We're using this feature to make Rift do the position calculation for us.
          
          -- Generate the timer text string.
          if remaining >= 60 then
            self.timer:SetText(string.format("%d:%02d:%02d", math.floor(remaining / 3600), math.floor(remaining / 60) % 60, math.floor(remaining) % 60))
          else
            self.timer:SetText(string.format("%2d.%d", math.floor(remaining) % 60, math.floor(((remaining-math.floor(remaining))*10))))
          end
          
          -- Update the width to avoid truncation.
          self.timer:SetWidth(self.timer:GetWidth())
        end
      end
    end
 end
  table.insert(activeBuffBars, bar)
  
  -- Show the bar and set the data.
  bar:SetVisible(true)
  bar.td = targetdetails
  bar.targetdetails = Inspect.Unit.Detail(bar.td)
  bar:SetBuff(buff, time)
  
  -- Attach it to the right spot.
  if not lastAttach then
    -- This is our first bar, so we're pinning it to the top center, slightly below the top edge of the screen. Most of the screen is taken by the standard UI and this is the least cluttered place we have.
    -- The downside is that lots of buffs will run down right into the player's field of view.
    bar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", WhimseyDots_Placement.x, WhimseyDots_Placement.y)
  else
    -- This isn't our first bar, so we pin it to the last bar.
    if lastAttach.debuff ~= buff.debuff then
      -- We're the first debuff, so add a gap between us and the last bar.
      bar:SetPoint("TOPLEFT", lastAttach, "BOTTOMLEFT", 0, 20)
    else
      -- Otherwise, we're flush with the last bar.
      bar:SetPoint("TOPLEFT", lastAttach, "BOTTOMLEFT")
    end
  end
  lastAttach = bar
end

local function refresh(time)
  -- Get the list of buffs from the player. Note that "player" can be replaced by "player.target" to turn this into a target's-buff-bar display.
  local bufflist = Inspect.Buff.List("player.target")
   -- Reset the buff bars and start entering buffs.
  if not bufflist then return end -- If we don't get anything, then we don't currently have information about the player. This may happen when the player is logging in or teleporting long distances.
  local targetdetails = Inspect.Unit.Lookup("player.target")
  --local targetdetails = Inspect.Unit.Detail("player.target")
  --if targetdetails.health == 0 then     
  --return
  --end
  
  local buffdetails = Inspect.Buff.Detail("player.target", bufflist) -- Retrieve the details for all of our buffs. If there were only certain buffs we cared about, we could filter them out before asking for detail.
  local playerdetails = Inspect.Unit.Lookup("player")--Inspect.Unit.Detail("player")
  StartBuffBars()
  
  
  -- We want to order buffs by their time remaining, splitting apart buffs and debuffs. We do the ordering here, in standard Lua code.
  local bbars = {}
  for id, buff in pairs(buffdetails) do
	if buff.caster == playerdetails then
		local skip = false
		for number, blackBuff in pairs(WhimseyDots_BlackList) do
			if (buff.name == blackBuff) then
				skip = true
				break
			end
		end
		if not skip then
				buff.id = id  -- Make a copy of the ID, because we'll need it
				table.insert(bbars, buff)
		end
	end
  end
  
  table.sort(bbars, function (a, b)
    if a.debuff ~= b.debuff then
      return b.debuff
    end
    
    if a.duration and b.duration then return a.remaining > b.remaining end
    if not a.duration and not b.duration then return false end
    return not a.duration
  end)
  
  -- Now that we have the ordering, we just add the bars one at a time. Done!
  for k, v in ipairs(bbars) do
    AddBuffBar(v, time,targetdetails )
  end
end

local resynch = false
local function tick()
  if resynch then
    -- A recalculation has been queued, so go ahead and recalculate.
    refresh(Inspect.Time.Frame())
    resynch = false
  else
    -- Just do a tick refresh
    local time = Inspect.Time.Frame()
    for _, v in ipairs(activeBuffBars) do
      v:Tick(time)
    end
  end
end

-- Events. See TrionDevelopmentTools for more detail on events.

-- A buff has changed! Rewrite the buff bars!
-- Likely, we could be more efficient by making use of this information about which buffs changed/added/removed.
-- At this current time we're not doing so. However, note that the buffs *do* provide enough information to do so.
-- Another thing we're not doing is triggering only on buffs changed on the player unit. We could be doing this as well, but, again, we're currently not.
-- As we might get *many* buff change events each frame, we're simply using a buff-change event to trigger a later refresh during the tick.
table.insert(Event.Buff.Add, {function () resynch = true end, "WhimseyDots", "refresh"})
table.insert(Event.Buff.Change, {function () resynch = true end, "WhimseyDots", "refresh"})
table.insert(Event.Buff.Remove, {function () resynch = true end, "WhimseyDots", "refresh"})
table.insert(Event.Unit.Add, {function () StartBuffBars() resynch = true end, "WhimseyDots", "refresh"})
table.insert(Event.Unit.Remove, {function () StartBuffBars() resynch = true end, "WhimseyDots", "refresh"})
--table.insert(Event.Unit.Change, {function (unit) dump(unit) end, "WhimseyDots", "refresh"})
table.insert(Event.Combat.Death, {function () StartBuffBars() resynch = true end, "WhimseyDots", "refresh"})


-- Our update event. This trigges every single frame.
table.insert(Event.System.Update.Begin, {tick, "WhimseyDots", "refresh"})


local context
local hasBeenClosed
local function config(commandline)
  
  	if not context then
	context = UI.CreateContext("WhimseyDotsConfigContext")
	hasBeenClosed = false
	-- Create our checkbox
	local window = UI.CreateFrame("RiftWindow", "SampleWindow", context)
	-- Put the checkbox in the middle of the screen
	window:SetPoint("TOPCENTER", UIParent, "TOPCENTER")
	-- Give a title to the window
	window:SetTitle("Change position")
	window:SetLayer(-1)
	window:SetHeight(500)
	window:SetWidth(320)
	
	
	
	function window.Event:MouseIn()
		print("Mouse in")
	end
	function window.Event:MouseOut()
		print("Mouse out")
	end
	
	local movableFrame = UI.CreateFrame("Frame", "MovableFrame", context)
	local movLocX = WhimseyDots_Placement.x
	local movLocY = WhimseyDots_Placement.y
	movableFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT",movLocX, movLocY)
	movableFrame:SetHeight(WhimseyDots_BarSize.y)
	movableFrame:SetWidth(WhimseyDots_BarSize.x)
	movableFrame:SetBackgroundColor(WhimseyDots_BarColorBack.r,WhimseyDots_BarColorBack.g,WhimseyDots_BarColorBack.b,WhimseyDots_BarColorBack.a)
	movableFrame:SetLayer(1)
	movableFrame:SetVisible(true)
	local movableFrameText = UI.CreateFrame("Text", "MovableFrameText", movableFrame)
	movableFrameText:SetPoint("TOPLEFT", movableFrame, "TOPLEFT")
	movableFrameText:SetText("Movable Frame")
	movableFrameText:SetFontSize(WhimseyDots_FontSize.text)
	movableFrameText:SetLayer(2)
	movableFrameText:SetFontColor(WhimseyDots_FontColor.r,WhimseyDots_FontColor.g,WhimseyDots_FontColor.b,WhimseyDots_FontColor.a)
	local movableFrameFrontColor = UI.CreateFrame("Frame", "MovableFrameFrontColor", movableFrame)
	movableFrameFrontColor:SetPoint("TOPLEFT", movableFrame, "TOPLEFT")
	movableFrameFrontColor:SetPoint("BOTTOMRIGHT", movableFrame, "BOTTOMRIGHT")
	movableFrameFrontColor:SetBackgroundColor(WhimseyDots_BarColorFront.r,WhimseyDots_BarColorFront.g,WhimseyDots_BarColorFront.b,WhimseyDots_BarColorFront.a)
	movableFrameFrontColor:SetWidth(WhimseyDots_BarSize.x / 2)
	movableFrameFrontColor:SetLayer(1)
	
	local movableFrameTimer = UI.CreateFrame("Text", "MovableFrameText", movableFrame)
	movableFrameTimer:SetPoint("TOPRIGHT", movableFrame, "TOPRIGHT")
	movableFrameTimer:SetText("10.1s")
	movableFrameTimer:SetFontSize(WhimseyDots_FontSize.timer)
	
	function movableFrame.Event:LeftDown()
		self.mouseDown = true
	end
	function movableFrame.Event:LeftUp()
		self.mouseDown = false
		WhimseyDots_Placement.x = movLocX
		WhimseyDots_Placement.y = movLocY
	end
	function movableFrame.Event:MouseMove()
		if(self.mouseDown) then
			local result = Inspect.Mouse()
			movLocX = result.x
			movLocY = result.y
			movableFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT",movLocX, movLocY)
		end
	end
	
	local frm_widthSelText = UI.CreateFrame("Text", "widthSelText", window)
	frm_widthSelText:SetPoint("TOPCENTER", window, "TOPCENTER", 0, 50)
	frm_widthSelText:SetText("Bar width")
	frm_widthSelText:SetHeight(frm_widthSelText:GetHeight())
	local frm_widthSelTextArea = UI.CreateFrame("Text", "widthSelTextArea", window)
	frm_widthSelTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 0, frm_widthSelText:GetHeight()+60)
	frm_widthSelTextArea:SetLayer(0)
	frm_widthSelTextArea:SetVisible(true)
	frm_widthSelTextArea:SetText(string.format("%d",WhimseyDots_BarSize.x))
	local frm_widthSel = UI.CreateFrame("RiftSlider", "widthSel", window)
	frm_widthSel:SetPoint("TOPCENTER", window, "TOPCENTER", 0, frm_widthSelText:GetHeight()+50)
	frm_widthSel:SetLayer(0)
	frm_widthSel:SetVisible(true)
	frm_widthSel:SetPosition(math.floor(WhimseyDots_BarSize.x / 6.4))
	function frm_widthSel.Event:SliderChange ()
		WhimseyDots_BarSize.x = frm_widthSel:GetPosition()* 6.4
		movableFrame:SetWidth(WhimseyDots_BarSize.x)
		--movableFrameFrontColor:SetWidth(WhimseyDots_BarSize.x / 2)
		frm_widthSelTextArea:SetText(string.format("%d",WhimseyDots_BarSize.x))
	end

	
	local frm_heightSelText = UI.CreateFrame("Text", "heightSelText", window)
	frm_heightSelText:SetPoint("TOPCENTER", window, "TOPCENTER", 0, 100)
	frm_heightSelText:SetText("Bar height")
	frm_heightSelText:SetHeight(frm_heightSelText:GetHeight())
	local frm_heightSelTextArea = UI.CreateFrame("Text", "heightSelTextArea", window)
	frm_heightSelTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 0, frm_heightSelText:GetHeight()+110)
	frm_heightSelTextArea:SetLayer(0)
	frm_heightSelTextArea:SetVisible(true)
	frm_heightSelTextArea:SetText(string.format("%d",WhimseyDots_BarSize.y))
	local frm_heightSel = UI.CreateFrame("RiftSlider", "heightSel", window)
	frm_heightSel:SetPoint("TOPCENTER", window, "TOPCENTER", 0, frm_heightSelText:GetHeight()+100)
	frm_heightSel:SetLayer(0)
	frm_heightSel:SetVisible(true)
	frm_heightSel:SetPosition(math.floor(WhimseyDots_BarSize.y))
	function frm_heightSel.Event:SliderChange ()
		WhimseyDots_BarSize.y = frm_heightSel:GetPosition()
		movableFrame:SetHeight(WhimseyDots_BarSize.y)
		frm_heightSelTextArea:SetText(string.format("%d",WhimseyDots_BarSize.y))
	end

	
	local frm_fontTextSelText = UI.CreateFrame("Text", "frm_fontTextSelText", window)
	frm_fontTextSelText:SetPoint("TOPCENTER", window, "TOPCENTER", -50, 150)
	frm_fontTextSelText:SetText("Spell Text Size")
	frm_fontTextSelText:SetHeight(frm_fontTextSelText:GetHeight())
	local frm_fontTextSelTextArea = UI.CreateFrame("RiftTextfield", "fontTextSelTextArea", window)
	frm_fontTextSelTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 50,150)
	frm_fontTextSelTextArea:SetLayer(0)
	frm_fontTextSelTextArea:SetVisible(true)
	frm_fontTextSelTextArea:SetBackgroundColor(0,0,0,1)
	frm_fontTextSelTextArea:SetWidth(20)
	frm_fontTextSelTextArea:SetText(string.format("%d",WhimseyDots_FontSize.text))	
	
	function frm_fontTextSelTextArea.Event:TextfieldChange()
		if not hasBeenClosed then
			local res = tonumber(frm_fontTextSelTextArea:GetText())
			if not res then
				--print("Error value not valid - Please pick a value between 1 and 40.")--frm_fontTextSelTextArea:SetText(string.format("%d", WhimseyDots_BarSize.x))
			else
				if res > 0 and res < 41 then
					WhimseyDots_FontSize.text = res
					movableFrameText:SetFontSize(WhimseyDots_FontSize.text)
				--else
					--print("Error value not valid - Please pick a value between 1 and 40.")
				end
			end
		end
	end
	
	local frm_fontTimerSelText = UI.CreateFrame("Text", "frm_fontTimerSelText", window)
	frm_fontTimerSelText:SetPoint("TOPCENTER", window, "TOPCENTER", -50, 172)
	frm_fontTimerSelText:SetText("Timer Text Size")
	frm_fontTimerSelText:SetHeight(frm_fontTimerSelText:GetHeight())
	local frm_fontTimerSelTextArea = UI.CreateFrame("RiftTextfield", "heightSelTextArea", window)
	frm_fontTimerSelTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 50,172)
	frm_fontTimerSelTextArea:SetLayer(0)
	frm_fontTimerSelTextArea:SetVisible(true)
	frm_fontTimerSelTextArea:SetBackgroundColor(0,0,0,1)
	frm_fontTimerSelTextArea:SetWidth(20)
	frm_fontTimerSelTextArea:SetText(string.format("%d",WhimseyDots_FontSize.timer))	
	
	function frm_fontTimerSelTextArea.Event:TextfieldChange()
		if not hasBeenClosed then
			local res = tonumber(frm_fontTimerSelTextArea:GetText())
			if not res then
				--print("Error value not valid - Please pick a value between 1 and 40.")--frm_fontTimerSelTextArea:SetText(string.format("%d", WhimseyDots_BarSize.x))
			else
				if res > 0 and res < 41 then
					WhimseyDots_FontSize.timer = res
					movableFrameTimer:SetFontSize(WhimseyDots_FontSize.timer)
				--else
					--print("Error value not valid - Please pick a value between 1 and 40.")
				end
			end
		end
	end
	
	--FRONT
	local frm_frontColorSelText = UI.CreateFrame("Text", "frm_frontColorSelText", window)
	frm_frontColorSelText:SetPoint("TOPCENTER", window, "TOPCENTER", 0, 190)
	frm_frontColorSelText:SetText("Bar Front Color")
	frm_frontColorSelText:SetHeight(frm_frontColorSelText:GetHeight())
	
	local frm_frontColorSelREDTextArea = UI.CreateFrame("RiftTextfield", "frm_frontColorREDTextArea", window)
	frm_frontColorSelREDTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", -48,210)
	frm_frontColorSelREDTextArea:SetLayer(0)
	frm_frontColorSelREDTextArea:SetVisible(true)
	frm_frontColorSelREDTextArea:SetBackgroundColor(0.7,0.2,0.2,1)
	frm_frontColorSelREDTextArea:SetWidth(30)
	frm_frontColorSelREDTextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorFront.r*255)))	
	
	local frm_frontColorSelGREENTextArea = UI.CreateFrame("RiftTextfield", "frm_frontColorGREENTextArea", window)
	frm_frontColorSelGREENTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", -16,210)
	frm_frontColorSelGREENTextArea:SetLayer(0)
	frm_frontColorSelGREENTextArea:SetVisible(true)
	frm_frontColorSelGREENTextArea:SetBackgroundColor(0.2,0.7,0.2,1)
	frm_frontColorSelGREENTextArea:SetWidth(30)
	frm_frontColorSelGREENTextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorFront.g*255)))
	
	local frm_frontColorSelBLUETextArea = UI.CreateFrame("RiftTextfield", "frm_frontColorBLUETextArea", window)
	frm_frontColorSelBLUETextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 16,210)
	frm_frontColorSelBLUETextArea:SetLayer(0)
	frm_frontColorSelBLUETextArea:SetVisible(true)
	frm_frontColorSelBLUETextArea:SetBackgroundColor(0.2,0.2,0.7,1)
	frm_frontColorSelBLUETextArea:SetWidth(30)
	frm_frontColorSelBLUETextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorFront.b*255)))
	
	local frm_frontColorSelALPHATextArea = UI.CreateFrame("RiftTextfield", "frm_frontColorALPHATextArea", window)
	frm_frontColorSelALPHATextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 48,210)
	frm_frontColorSelALPHATextArea:SetLayer(0)
	frm_frontColorSelALPHATextArea:SetVisible(true)
	frm_frontColorSelALPHATextArea:SetBackgroundColor(0.2,0.2,0.2,1)
	frm_frontColorSelALPHATextArea:SetWidth(30)
	frm_frontColorSelALPHATextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorFront.a*255)))

	--BACK
	local frm_backColorSelText = UI.CreateFrame("Text", "frm_backColorSelText", window)
	frm_backColorSelText:SetPoint("TOPCENTER", window, "TOPCENTER", 0, 240)
	frm_backColorSelText:SetText("Bar Back Color")
	frm_backColorSelText:SetHeight(frm_backColorSelText:GetHeight())
	
	local frm_backColorSelREDTextArea = UI.CreateFrame("RiftTextfield", "frm_backColorREDTextArea", window)
	frm_backColorSelREDTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", -48,260)
	frm_backColorSelREDTextArea:SetLayer(0)
	frm_backColorSelREDTextArea:SetVisible(true)
	frm_backColorSelREDTextArea:SetBackgroundColor(0.7,0.2,0.2,1)
	frm_backColorSelREDTextArea:SetWidth(30)
	frm_backColorSelREDTextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorBack.r*255)))	
	
	local frm_backColorSelGREENTextArea = UI.CreateFrame("RiftTextfield", "frm_backColorGREENTextArea", window)
	frm_backColorSelGREENTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", -16,260)
	frm_backColorSelGREENTextArea:SetLayer(0)
	frm_backColorSelGREENTextArea:SetVisible(true)
	frm_backColorSelGREENTextArea:SetBackgroundColor(0.2,0.7,0.2,1)
	frm_backColorSelGREENTextArea:SetWidth(30)
	frm_backColorSelGREENTextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorBack.g*255)))
	
	local frm_backColorSelBLUETextArea = UI.CreateFrame("RiftTextfield", "frm_backColorBLUETextArea", window)
	frm_backColorSelBLUETextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 16,260)
	frm_backColorSelBLUETextArea:SetLayer(0)
	frm_backColorSelBLUETextArea:SetVisible(true)
	frm_backColorSelBLUETextArea:SetBackgroundColor(0.2,0.2,0.7,1)
	frm_backColorSelBLUETextArea:SetWidth(30)
	frm_backColorSelBLUETextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorBack.b*255)))
	
	local frm_backColorSelALPHATextArea = UI.CreateFrame("RiftTextfield", "frm_backColorALPHATextArea", window)
	frm_backColorSelALPHATextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 48,260)
	frm_backColorSelALPHATextArea:SetLayer(0)
	frm_backColorSelALPHATextArea:SetVisible(true)
	frm_backColorSelALPHATextArea:SetBackgroundColor(0.2,0.2,0.2,1)
	frm_backColorSelALPHATextArea:SetWidth(30)
	frm_backColorSelALPHATextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorBack.a*255)))
	
	--TEXT
	local frm_fontColorSelText = UI.CreateFrame("Text", "frm_fontColorSelText", window)
	frm_fontColorSelText:SetPoint("TOPCENTER", window, "TOPCENTER", 0, 290)
	frm_fontColorSelText:SetText("Font Color")
	frm_fontColorSelText:SetHeight(frm_fontColorSelText:GetHeight())
	
	local frm_fontColorSelREDTextArea = UI.CreateFrame("RiftTextfield", "frm_fontColorREDTextArea", window)
	frm_fontColorSelREDTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", -48, 310)
	frm_fontColorSelREDTextArea:SetLayer(0)
	frm_fontColorSelREDTextArea:SetVisible(true)
	frm_fontColorSelREDTextArea:SetBackgroundColor(0.7,0.2,0.2,1)
	frm_fontColorSelREDTextArea:SetWidth(30)
	frm_fontColorSelREDTextArea:SetText(string.format("%d",math.floor(WhimseyDots_FontColor.r*255)))	
	
	local frm_fontColorSelGREENTextArea = UI.CreateFrame("RiftTextfield", "frm_fontColorGREENTextArea", window)
	frm_fontColorSelGREENTextArea:SetPoint("TOPCENTER", window, "TOPCENTER", -16,310)
	frm_fontColorSelGREENTextArea:SetLayer(0)
	frm_fontColorSelGREENTextArea:SetVisible(true)
	frm_fontColorSelGREENTextArea:SetBackgroundColor(0.2,0.7,0.2,1)
	frm_fontColorSelGREENTextArea:SetWidth(30)
	frm_fontColorSelGREENTextArea:SetText(string.format("%d",math.floor(WhimseyDots_FontColor.g*255)))
	
	local frm_fontColorSelBLUETextArea = UI.CreateFrame("RiftTextfield", "frm_fontColorBLUETextArea", window)
	frm_fontColorSelBLUETextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 16,310)
	frm_fontColorSelBLUETextArea:SetLayer(0)
	frm_fontColorSelBLUETextArea:SetVisible(true)
	frm_fontColorSelBLUETextArea:SetBackgroundColor(0.2,0.2,0.7,1)
	frm_fontColorSelBLUETextArea:SetWidth(30)
	frm_fontColorSelBLUETextArea:SetText(string.format("%d",math.floor(WhimseyDots_FontColor.b*255)))
	
	local frm_fontColorSelALPHATextArea = UI.CreateFrame("RiftTextfield", "frm_fontColorALPHATextArea", window)
	frm_fontColorSelALPHATextArea:SetPoint("TOPCENTER", window, "TOPCENTER", 48,310)
	frm_fontColorSelALPHATextArea:SetLayer(0)
	frm_fontColorSelALPHATextArea:SetVisible(true)
	frm_fontColorSelALPHATextArea:SetBackgroundColor(0.2,0.2,0.2,1)
	frm_fontColorSelALPHATextArea:SetWidth(30)
	frm_fontColorSelALPHATextArea:SetText(string.format("%d",math.floor(WhimseyDots_FontColor.a*255)))
	



	
	function SetColor(frame, palette, color)
		if not hasBeenClosed then
			local res = tonumber(frame:GetText())
			if res then		
				if res > -1 and res < 256 then
					if color == "red" then
						palette.r = res/255
					end
					if color == "green" then
						palette.g = res/255
					end
					if color == "blue" then
						palette.b = res/255
					end
					if color == "alpha" then
						palette.a = res/255
					end
					movableFrameFrontColor:SetBackgroundColor(WhimseyDots_BarColorFront.r,WhimseyDots_BarColorFront.g,WhimseyDots_BarColorFront.b,WhimseyDots_BarColorFront.a)
					movableFrame:SetBackgroundColor(WhimseyDots_BarColorBack.r,WhimseyDots_BarColorBack.g,WhimseyDots_BarColorBack.b,WhimseyDots_BarColorBack.a)
					movableFrameText:SetFontColor(WhimseyDots_FontColor.r, WhimseyDots_FontColor.g, WhimseyDots_FontColor.b, WhimseyDots_FontColor.a)
				end
			end
		else
			print("Frame is closed...")
		end
	end
	--FRONT
	function frm_frontColorSelREDTextArea.Event:TextfieldChange()
		SetColor(frm_frontColorSelREDTextArea,WhimseyDots_BarColorFront, "red")
	end
	function frm_frontColorSelGREENTextArea.Event:TextfieldChange()
		SetColor(frm_frontColorSelGREENTextArea, WhimseyDots_BarColorFront, "green")
	end
	function frm_frontColorSelBLUETextArea.Event:TextfieldChange()
		SetColor(frm_frontColorSelBLUETextArea, WhimseyDots_BarColorFront, "blue")
	end
	function frm_frontColorSelALPHATextArea.Event:TextfieldChange()
		SetColor(frm_frontColorSelALPHATextArea, WhimseyDots_BarColorFront, "alpha")
	end
	--BACK
	function frm_backColorSelREDTextArea.Event:TextfieldChange()
		SetColor(frm_backColorSelREDTextArea,WhimseyDots_BarColorBack, "red")
	end
	function frm_backColorSelGREENTextArea.Event:TextfieldChange()
		SetColor(frm_backColorSelGREENTextArea, WhimseyDots_BarColorBack, "green")
	end
	function frm_backColorSelBLUETextArea.Event:TextfieldChange()
		SetColor(frm_backColorSelBLUETextArea, WhimseyDots_BarColorBack, "blue")
	end
	function frm_backColorSelALPHATextArea.Event:TextfieldChange()
		SetColor(frm_backColorSelALPHATextArea, WhimseyDots_BarColorBack, "alpha")
	end
	--FONT
	function frm_fontColorSelREDTextArea.Event:TextfieldChange()
		SetColor(frm_fontColorSelREDTextArea,WhimseyDots_FontColor, "red")
	end
	function frm_fontColorSelGREENTextArea.Event:TextfieldChange()
		SetColor(frm_fontColorSelGREENTextArea, WhimseyDots_FontColor, "green")
	end
	function frm_fontColorSelBLUETextArea.Event:TextfieldChange()
		SetColor(frm_fontColorSelBLUETextArea, WhimseyDots_FontColor, "blue")
	end
	function frm_fontColorSelALPHATextArea.Event:TextfieldChange()
		SetColor(frm_fontColorSelALPHATextArea, WhimseyDots_FontColor, "alpha")
	end
	
	--local checkBoxBlackList = UI.CreateFrame("RiftCheckbox
	
	local escText = UI.CreateFrame("Text", "EscText", window)
	escText:SetText("Press ESCAPE if you have been editing colors, else the game will think you are editing even after this dialog has been closed.")
	escText:SetFontSize(15)
	escText:SetPoint("TOPCENTER", window, "TOPCENTER", 10, 350)
	escText:SetWidth(window:GetWidth()-50)
	escText:SetWordwrap(true)
	
	local btn = UI.CreateFrame("RiftButton", "Close", window)
	btn:SetText("Close")
	btn:SetPoint("TOPCENTER", window, "TOPCENTER", 0, 450)
	btn:SetVisible(true)
	btn:SetLayer(0)
	function btn.Event:LeftPress()		
		self.mouseDown = false
		hasBeenClosed = true
		context:SetVisible(false)
	end
	else
		hasBeenClosed = false
		-- frm_widthSelTextArea:SetText(string.format("%d",WhimseyDots_BarSize.x))
		-- frm_heightSelTextArea:SetText(string.format("%d",WhimseyDots_BarSize.y))
		-- frm_fontTextSelTextArea:SetText(string.format("%d",WhimseyDots_FontSize.text))	
		-- frm_fontTimerSelTextArea:SetText(string.format("%d",WhimseyDots_FontSize.timer))	
		-- frm_frontColorSelREDTextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorFront.r*255)))	
		-- frm_frontColorSelGREENTextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorFront.g*255)))
		-- frm_frontColorSelBLUETextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorFront.b*255)))
		-- frm_frontColorSelALPHATextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorFront.a*255)))
		-- frm_backColorSelREDTextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorBack.r*255)))	
		-- frm_backColorSelGREENTextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorBack.g*255)))
		-- frm_backColorSelBLUETextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorBack.b*255)))
		-- frm_backColorSelALPHATextArea:SetText(string.format("%d",math.floor(WhimseyDots_BarColorBack.a*255)))
		-- frm_fontColorSelREDTextArea:SetText(string.format("%d",math.floor(WhimseyDots_FontColor.r*255)))	
		-- frm_fontColorSelGREENTextArea:SetText(string.format("%d",math.floor(WhimseyDots_FontColor.g*255)))
		-- frm_fontColorSelBLUETextArea:SetText(string.format("%d",math.floor(WhimseyDots_FontColor.b*255)))
		-- frm_fontColorSelALPHATextArea:SetText(string.format("%d",math.floor(WhimseyDots_FontColor.a*255)))
		context:SetVisible(true)
	end
end
table.insert(Command.Slash.Register("WhimseyDots"), {config, "WhimseyDots", "config"})